/*	
 *	swipe() :: Swipe action using CSS transforms with jQuery position fallback
 *
 *	Copyright by Lazaworx
 *	http://www.lazaworx.com
 *	Author: Laszlo Molnar
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Usage: $(element).swipe( options );
 */
 
;(function($, $window, $document, undefined) {
	'use strict';
	
	$.fn.swipe = function(options) {
		
		options = $.extend({
						onSwipedLeft:			false,			// Swiped left event
						onSwipedRight:			false,			// Swiped right event
						onSwipedUp:				false,			// Swiped up event
						onSwipedDown:			false,			// Swiped down event
						onSmallSwipe:			false,			// Small swipe (< threshold px)
						onSwipeStart:			false,			// Start swiping event
						onPinchZoom:			false,			// Pinch zoomed event
						onClick:				false,			// Clicked
						onDoubleClick:			false,			// Doble-clicked
						onFinished:				false,			// Swiped out - can remove
						threshold:				20,				// Small swipe threshold
						clickThreshold:			150,			// Timespan to tell apart single and double click
						doubleClickThreshold:	200,
						noSwipeInside:			'.caption',		// Avoid swipe inside this container
						stopAutoPano:			true,			// Stops autopano move, and doesn't trigger click if done so
						swipeoutSpeed:			300,			// Swipe out speed in ms
						overThreshold:			0.25			// Oversized pictures must moved beyond (this * screen width) to trigger swipe 
					}, options);
		
		// Getting coordinates of a touch or mouse event
		var	getDim = function(el) {
					return [ el.outerWidth(), el.outerHeight() ];
				},

			getPos = function(e) {
					if (e.touches) {
						if (e.touches.length > 1) {
							// Two finger event
							return [ e.touches[0].clientX, e.touches[0].clientY, e.touches[1].clientX, e.touches[1].clientY ];
						}
						// Single finger event
						return [ e.touches[0].clientX, e.touches[0].clientY ];
					}
					// mouse event
					return (e.clientX === null)? null : [ e.clientX, e.clientY ];
				};
				
		return this.each(function() {
				
				var self = $(this),							// target element
					cont = self.parent(),					// container or clipping window
					ns = self.data('lsw_ns') ||				// namespace 
						('lsw_' + Math.floor(Math.random() * 10000)),
					hasStarted = false,						// flag to avoid double start on touch devices
					selfDim,								// item dimensions
					clipDim,								// clip dimensions
					rto,									// window resize update timeout
					startTime,								// start time
					endTime,								// end time
					startPos,								// initial position Array[2..4]
					startD,									// initial distance of two finger events
					cPos,									// Current position
					mx,										// original matrix
					started = false,						// ongoing swipe
					ended = true,							// ended swiping
					swipeHandled = true,					// swipe event already fired
					dontSwipeUp = true,						// limit vertical swipe
					dontSwipeDown = true,
					canConstrainHorz = false,				// Can constrain to horizontal
					nonSwipeable = true,					// target element swipeable?
					clickedOnce = false,					// Check for double click
					hadAutoPano = false,					// Remembering if autopano was on
					chkDoubleClick = null,					// Checking double-click (timeout)
					timer = null,
					media = self.hasClass('audio') || self.hasClass('video'), 
					
					// Handling swipe start event
					handleSwipeStart = function(e) {
							if (typeof options.onSwipeStart === FUNCTION) {
								options.onSwipeStart.call(self[0], e);
							}
						},
						
					// Handling swipeout movement finished event
					handleFinished = function(e) {
							//log('handleFinished');
							clearTimeout(timer);
							if (typeof options.onFinished === FUNCTION) {
								options.onFinished.call(self[0], e);
							}
						},
						
					// Handling small swipe
					handleSmallSwipe = function(e) {
							//log('handleSmallSwipe');
							unregisterEvents(e);
							if (typeof options.onSmallSwipe === FUNCTION) {
								options.onSmallSwipe.call(self[0], e);
							}
						},
						
					// Handling click event
					handleClick = function(e) {
							//log('handleClick');
							unregisterEvents(e);
							if (!hadAutoPano && typeof options.onClick === FUNCTION) {
								// Avoid click when stopping panomove consumed the event
								options.onClick.call(self[0], e);
							}
						},
						
					// Handling doubleclick event
					handleDoubleClick = function(e) {
							//log('handleDoubleClick');
							unregisterEvents(e);
							if (typeof options.onDoubleClick === FUNCTION) {
								options.onDoubleClick.call(self[0], e);
							}
						},
						
					// Handling swipe		
					handleSwipedDown = function(e) {
							//log('handleSwipe:'+(right?'right':'left'));
							if (!swipeHandled) {
								unregisterEvents(e);
								swipeHandled = true;
								// Swiped down
								options.onSwipedDown.call(self[0], e);
							}
						},
							
					// Handling swipe		
					handleSwipedUp = function(e) {
							//log('handleSwipe:'+(right?'right':'left'));
							if (!swipeHandled) {
								unregisterEvents(e);
								swipeHandled = true;
								// Swiped up
								options.onSwipedUp.call(self[0], e);
							}
						},
							
					// Handling swipe		
					handleSwipedRight = function(e) {
							//log('handleSwipe:'+(right?'right':'left'));
							if (!swipeHandled) {
								unregisterEvents(e);
								swipeHandled = true;
								// Swiped right
								options.onSwipedRight.call(self[0], e);
							}
						},
							
					// Handling swipe		
					handleSwipedLeft = function(e) {
							//log('handleSwipe:'+(right?'right':'left'));
							if (!swipeHandled) {
								unregisterEvents(e);
								swipeHandled = true;
								// Swiped left
								options.onSwipedLeft.call(self[0], e);
							}
						},
						
					// Handling pinch zoom
					handlePinchZoom = function(e) {
							//log('handlePinchZoom:'+(right?'right':'left'));
							if (!swipeHandled) {
								unregisterEvents(e);
								swipeHandled = true;
								// Swiped left
								options.onPinchZoom.call(self[0], e);
							}
						},
							
					// Handling right click event
					handleRightClick = function(e) {
							//log('handleRightClick');
							unregisterEvents(e);
							if (typeof options.onRightClick === FUNCTION) {
								options.onRightClick.call(self[0], e);
							}
						},
					
					// Registering events
					registerEvents = function(e) { 
							//log('registerEvents');
							if (e.type === 'mousedown') {
								self.on('mouseup.' + ns + ' mouseout.' + ns, swipeEnd);
								$document.on('mousemove.' + ns, swipeMove);
							} else if (e.type === TOUCH.START) {
								self.on(TOUCH.END + '.' + ns, swipeEnd);
							}
							clickedOnce = false;
						},
						
					unregisterEvents = function(e) {
							//log('unregisterEvents');
							if (e.type === 'mouseup' || e.type === 'mouseout') {
								self.off('mouseup.' + ns + ' mouseout.' + ns);
								$document.off('mousemove.' + ns);
							} else if (e.type === TOUCH.END) {
								self.off(TOUCH.END + '.' + ns);
							}
						},
						
					// Getting distance of two finger events
					getDistance = function(p) {
							return Math.hypot(p[2] - p[0], p[3] - p[1]);
						},

					// Move to
					transformTo = function(p, op, speed, doneFn) {
							var css = {},
								dx,
								dy,
								sc;
							
							if (p.length === 4) {
								// Move plus scale; 2 finger move
								sc = getDistance(p) / startD;
								// Midpoint
								dx = (p[0] - startPos[0] + p[2] - startPos[2]) / 2;
								dy = (p[1] - startPos[1] + p[3] - startPos[3]) / 2;
							} else {
								// Move only
								dx = p[0] - startPos[0];
								dy = p[1] - startPos[1];
								sc = 1;
							}
							
							css.transform = 'matrix(' + (mx[0] * sc) + ',' + mx[1] + ',' + mx[2] + ',' + (mx[3] * sc) + ',' + (mx[4] + dx) + ',' + (mx[5] + dy) + ')';
							
							if (typeof speed !== UNDEF && speed) {
								css.transition = 'transform ' + speed + 'ms ease-out';
								if (op !== null) {
									css.transition += ',opacity ' + speed + 'ms ease-out';
									css.opacity = op;
								}
							} else {
								css.transition = 'none';
							}
							
							if (typeof doneFn === FUNCTION) {
								self.one('transitionend', doneFn);
							}
							
							self.css(css);
						},
				
					// Starting swipe
					swipeStart = function(e) {
							//log('swipeStart:' + e.type);
							
							//log('Touch start, fingers: ' + (e.originalEvent.touches? e.originalEvent.touches.length : 0));
							if (pageZoomed() ||
								e.originalEvent.touches && e.originalEvent.touches.length > 2 ||
								(options.noSwipeInside && $(e.target).closest(options.noSwipeInside).length) ||
								e.target.nodeName === 'A' ||
								e.target.nodeName === 'INPUT' ||
								e.target.nodeName === 'BUTTON' ||
								e.target.nodeName === 'AUDIO' || 
								(e.target.nodeName === 'VIDEO' && 
									(!e.target.paused ||
										e.target.controls && 
										(ISIOSDEVICE || e.offsetY > ($(e.target).height() - 60))
									)
								)
							) {
								// Don't swipe buttons and links and other non-swipeable elements
								// No swiping on iOS video player
								nonSwipeable = true;
								return true;
							}
							
							if (e.type === 'mousedown' && e.which !== 1) {									
								// Right click
								if (options.stopAutoPano && cont.data('panomove')) {
									self.trigger('autopanoStop');
								}
								handleRightClick(e);
								return true;
							}
							
							if (hasStarted || clickedOnce) {
								// No mouseup / touchend registered yet
								return false;
							}
							
							if (e.type !== 'touchstart') {
								// Cancel bubbling, but not break long tap
								e.preventDefault();
							}
							
							hasStarted = true;
							nonSwipeable = false;
							ended = false;
							clickedOnce = false;
							
							if (options.stopAutoPano && cont.data('panomove')) {
								self.trigger('autopanoStop');
								hadAutoPano = true;
							} else {
								hadAutoPano = false;
							}
							
							self.data({
									scrolling: 		true,
									swipeEnded:		false,
									taplength:		0
								});
							
							startTime = new Date().getTime();
							cPos = startPos = getPos(e.originalEvent);
							if (startPos.length === 4) {
								startD = getDistance(startPos);
							}
							clipDim = getDim(cont);
							selfDim = getDim(self);
							
							canConstrainHorz = clipDim[1] >= selfDim[1];
							
							if (canConstrainHorz) {
								dontSwipeUp = typeof options.onSwipedUp !== FUNCTION;
								dontSwipeDown = typeof options.onSwipedDown !== FUNCTION;
							} else {
								dontSwipeUp = dontSwipeDown = false;
							}
							
							mx = self.getMatrix();
							
							registerEvents(e);
							
							handleSwipeStart(e);
							
							swipeHandled = false;
							
							// Do not cancel long tap
							return e.type === 'touchstart';
						},
					
					// Moving in progress
					swipeMove = function(e) {
							//log('swipeMove:' + e.type + ' [' + e.originalEvent.touches.length + ']');
							
							if (ended || nonSwipeable ||
								e.originalEvent.touches && e.originalEvent.touches.length > 2) {
								// not active, prohibited area or three finger touch
								return true;
							}
							
							e.preventDefault();
							
							cPos = getPos(e.originalEvent);
							
							if (startPos === null || startPos.length < cPos.length) {
								for (var i = startPos.length || 0; i < cPos.length; i++) {
									startPos[i] = cPos[i];
								}
								startD = getDistance(cPos);
							}
														
							if (dontSwipeUp && dontSwipeDown) {
								// Horizontal-only
								cPos[1] = startPos[1];
							} else {
								var dx = cPos[0] - startPos[0],
									dy = cPos[1] - startPos[1];
								// Can move both axes
								if (canConstrainHorz && (Math.abs(dx) > 30 || (Math.abs(dx) > 8 && Math.abs(dx / dy) > 0.25))) {
									// Change to horizontal-only swipe
									dontSwipeUp = dontSwipeDown = true;
								}
								
								if (dontSwipeUp && cPos[1] < startPos[1] ||
									dontSwipeDown && cPos[1] > startPos[1]) {
									cPos[1] = startPos[1];
								}
							}
							
							transformTo(cPos);
							return false;
						},
					
					// Cancel swipe (e.g. moved out of the screen)
					swipeCancel = function(e) {
							//log('swipeCancel:' + e.type);
							
							ended = true;
							
							hasStarted = false;
							
							unregisterEvents(e);
							
							setTimeout(function() { 
									self.data({
											scrolling:	 	false,
											swipeEnded:		true
										});
								}, 100);
							
							transformTo(startPos, null, 100);
							
							return false;
						},
						
					// Stopped moving or clicked : mouse/touch up
					swipeEnd = function(e) {
							//log('swipeEnd:' + e.type);
							if (nonSwipeable) {
								// Prohibited area
								return true;
							}
							
							e.preventDefault();
							
							if (chkDoubleClick) {
								clearTimeout(chkDoubleClick);
								chkDoubleClick = null;
							}
							
							hasStarted = false;
							ended = true;
							endTime = new Date().getTime();
							
							if (cPos.length === 4) {
								handlePinchZoom(e);
							} else {
								var dx = cPos[0] - startPos[0],
									dy = cPos[1] - startPos[1];
								
								if (((endTime - startTime) > 50) && (Math.hypot(dx, dy) > options.threshold)) {
									//log('swiped: ' + (endTime - startTime) + 'ms ' + getDistance(diff) + 'px');
									// Not a click, because it took longer or the distance is larger than the threshold
									var speed = 1 + options.swipeoutSpeed / (endTime - startTime);
									
									self.data({
											scrolling:	 	true,
											swipeEnded:		true,
											taplength:		endTime - startTime
										});
									
									if (dontSwipeUp && dontSwipeDown) {
										// Constrain horizontally
										var targetDx = dx * speed;
										
										if ((clipDim[0] >= selfDim[0]) ||
												Math.abs(dx) > 0 && (
													((mx[4] + targetDx) > clipDim[0] * (options.overThreshold - 0.5)) ||
													((mx[4] + targetDx + selfDim[0]) < clipDim[0] * (0.5 - options.overThreshold))
												)
											) {
											// Swiped => move and fade out
											transformTo([ cPos[0] + targetDx, startPos[1] ], 0, options.swipeoutSpeed, handleFinished);
											
											if (dx > 0) {
												handleSwipedRight(e);
											} else {
												handleSwipedLeft(e);
											}
											
										} else {
											// Panoramic :: Hasn't reached the edge yet
											transformTo([ cPos[0] + targetDx, startPos[1] ], null, options.swipeoutSpeed);
											unregisterEvents(e);
										}
									
									} else {
										// No constrain
										var vertical = Math.abs(dy) > 1.5 * Math.abs(dx), // max 30 degrees
											targetDx = dx * speed,
											targetDy = dy * speed;
										
										if (vertical && (
												selfDim[1] > clipDim[1] && ( 
													(dy > 0 && (mx[5] + targetDy) <= clipDim[1] * (options.overThreshold - 0.5)) ||
													(dy < 0 && (mx[5] + targetDy + selfDim[1]) >= clipDim[1] * (0.5 - options.overThreshold))
												)
											) ||
											!vertical && (
												selfDim[0] > clipDim[0] && ( 
													(dx > 0 && (mx[4] + targetDx) <= clipDim[0] * (options.overThreshold - 0.5)) ||
													(dx < 0 && (mx[4] + targetDx + selfDim[0]) >= clipDim[0] * (0.5 - options.overThreshold))
												)
											)) { 
											// Hasn't reached the edge yet; continue scrolling
											transformTo([ cPos[0] + targetDx, cPos[1] + targetDy ], null, options.swipeoutSpeed);
											unregisterEvents(e);
										} else {
											// Swiped over the boundaries										
											if (typeof options.onSwipedDown === FUNCTION && vertical && dy > 0) {
												// Swiped down
												transformTo([ startPos[0], cPos[1] + targetDy ], 0, options.swipeoutSpeed, handleFinished);
												handleSwipedDown(e);
											} else if (typeof options.onSwipedUp === FUNCTION && vertical && dy < 0) {
												// Swiped up
												transformTo([ startPos[0], cPos[1] + targetDy ], 0, options.swipeoutSpeed, handleFinished);
												handleSwipedUp(e);
											} else {
												// Horizontal swipe or vertical but no up or down swipe handler
												transformTo([ cPos[0] + targetDx, cPos[1] + targetDy ], 0, options.swipeoutSpeed, handleFinished);
												if (dx > 0) {
													handleSwipedRight(e);
												} else {
													handleSwipedLeft(e);
												}
											}
										}
									}	
								
									clickedOnce = false;
								
								} else if (dx || dy) {
									// Small swipe: no click triggered
									handleSmallSwipe(e);
									transformTo(startPos, null, 100);
									clickedOnce = false;
									
								} else if (options.onDoubleClick) {
									// Has doubleclick handler event
									if (clickedOnce) {
										// One click already registered
										if ((new Date().getTime() - endTime) < options.doubleClickThreshold) {
											// Within threshold: Double click
											handleDoubleClick(e);
										} else {
											// Over threshold
											handleClick(e);
										}
										clickedOnce = false;
											
									} else { 
										// Wait if second click happens
										clickedOnce = true;
										// Triggering single click if no second click comes
										chkDoubleClick = setTimeout(function(e) {
												handleClick(e);
												clickedOnce = false;
											}, options.doubleClickThreshold + 10, e);
									}
									
								} else {
									// Handling single clicks only if no autopano was on
									handleClick(e);
								}
							}
							
							return false;
						},
					
					noAction = function(e) {
							e.preventDefault();
							return false;
						},
					
					// Tearing off the swipe handler
					removeSwipe = function() {
							setTimeout(function() { 
									self.data('scrolling', false);
								}, 20);
							self.removeAttr('draggable');
							self.add($document).off('.' + ns);
						};
					
					
				// clean up events that may hang around (in theory this should never happen)
				self.add($document).off('.' + ns);
				
				// Storing namespace
				self.data('lsw_ns', ns);
				
				// Need to be updated about the window size
				$window.on('resize.' + ns, function() {
						clearTimeout(rto);
						rto = setTimeout(function() {
							clipDim = getDim(cont);		
						}, 50);
					});
				
				self.attr('draggable', 'false')
					.on(TOUCH.START + '.' + ns + ' dragstart.' + ns + ' mousedown.' + ns, swipeStart)
					//.on(TOUCH.CANCEL + '.' + ns + ' dragcancel.' + ns, swipeCancel)
					.on(TOUCH.MOVE + '.' + ns + ' drag.' + ns, swipeMove)
					// External remove request
					.on('removeSwipe.' + ns, removeSwipe)
					// Do not select
					.on('selectstart.' + ns, noAction);
				
				//self.logEvents();
		
			});
		};
			
})(jQuery, jQuery(window), jQuery(document));
